<?php

namespace Drupal\screenshot_one\Plugin\AiAutomatorType;

use Drupal\ai_automator\Attribute\AiAutomatorType;
use Drupal\ai_automator\PluginInterfaces\AiAutomatorTypeInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\screenshot_one\Plugin\AiAutomatorType\ScreenshotBase;

/**
 * The rules for a screenshot media field.
 */
#[AiAutomatorType(
  id: 'screenshot_one_media',
  label: new TranslatableMarkup('Screenshot One Media'),
  field_rule: 'entity_reference',
  target: 'media',
)]
class ScreenshotMedia extends ScreenshotBase implements AiAutomatorTypeInterface, ContainerFactoryPluginInterface {

  /**
   * {@inheritDoc}
   */
  public $title = 'Screenshot One Media';

  /**
   * {@inheritDoc}
   */
  public function extraFormFields(ContentEntityInterface $entity, FieldDefinitionInterface $fieldDefinition, FormStateInterface $formState, array $defaultValues = []) {
    $form = parent::extraFormFields($entity, $fieldDefinition, $formState, $defaultValues);
    $form['automator_media'] = [
      '#type' => 'select',
      '#title' => 'Media Type',
      '#options' => $this->getFileHelper()->getMediaBundles(),
      '#description' => $this->t('The media type to generate'),
      '#default_value' => $defaultValues['automator_media'] ?? NULL,
    ];

    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function storeValues(ContentEntityInterface $entity, array $values, FieldDefinitionInterface $fieldDefinition, $automatorConfig) {
    $fileHelper = $this->getFileHelper();
    $params = $this->getParams($automatorConfig);

    // Transform string to boolean.
    $mediaEntities = [];

    // Successful counter, to only download as many as max.
    $successFul = 0;
    foreach ($values as $value) {
      // Get filename from previewURL.
      $fileName = 'screenshot_' . md5($value) . '.jpg';
      // Get the binary.
      $fileString = $this->screenshotOne->screenshotUrl($value, $params);
      // Set a name.
      $name = substr($this->t('Screenshot of @url', ['@url' => $value]), 0, 250);
      // Generate the media.
      $media = $fileHelper->generateMediaImageFromFile($fileName, $fileString, $automatorConfig['media'], $name, [
        'alt' => $name,
        'title' => $name,
      ]);

      if ($media) {
        $mediaEntities[] = $media;

        $successFul++;
        // If we have enough images, give up.
        if ($successFul == $fieldDefinition->getFieldStorageDefinition()->getCardinality()) {
          break;
        }
      }
    }

    // Then set the value.
    $entity->set($fieldDefinition->getName(), $mediaEntities);
  }

}
