<?php

namespace Drupal\screenshot_one;

use Drupal\ai_automator\Exceptions\AiAutomatorRequestErrorException;
use Drupal\screenshot_one\Form\ScreenshotConfigForm;
use Drupal\Core\Config\ConfigFactory;
use Drupal\key\KeyRepository;
use ScreenshotOne\Sdk\Client;
use ScreenshotOne\Sdk\TakeOptions;

/**
 * Screenshot One API.
 */
class ScreenshotOne {

  /**
   * API Key.
   */
  private string $apiKey;

  /**
   * Secret Key.
   */
  private string $secretKey;

  /**
   * Constructs a new SERP object.
   *
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   *   The config factory.
   * @param \Drupal\key\KeyRepository $keyRepository
   *   The key repository.
   */
  public function __construct( ConfigFactory $configFactory, KeyRepository $keyRepository) {
    $config = $configFactory->get(ScreenshotConfigForm::CONFIG_NAME);
    // Load API key.
    $key = $config->get('api_key') ?? '';
    if ($key) {
      $this->apiKey = $keyRepository->getKey($key)->getKeyValue();
    }
    $key = $config->get('secret_key') ?? '';
    if ($key) {
      $this->secretKey = $keyRepository->getKey($key)->getKeyValue();
    }
  }

  /**
   * Screenshot API Call.
   *
   * @param string $uri
   *   The uri to screenshot.
   * @param array $parameters
   *   The parameters for the call.
   *
   * @return string
   *   A binary for an image.
   */
  public function screenshotUrl($url, array $parameters) {
    if (empty($url) || !filter_var($url, FILTER_VALIDATE_URL)) {
      throw new AiAutomatorRequestErrorException('You have to give an url');
    }
    $client = new Client($this->apiKey, $this->secretKey);
    $options = TakeOptions::url($url);
    $options->fullPage($parameters['full_page'] ?? TRUE);
    $options->delay($parameters['delay'] ?? 0);
    $options->viewportWidth($parameters['width'] ?? 1280);
    $options->viewportHeight($parameters['height'] ?? 800);
    $options->format($parameters['format'] ?? 'jpg');
    $options->imageQuality($parameters['image_quality'] ?? 80);
    if (!empty($parameters['selector'])) {
      $options->selector($parameters['selector']);
    }
    if (!empty($parameters['geolocation_latitude'])) {
      $options->geolocationLatitude($parameters['geolocation_latitude']);
    }
    if (!empty($parameters['geolocation_longitude'])) {
      $options->geolocationLongitude($parameters['geolocation_longitude']);
    }

    return $client->take($options);
  }

}
